"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = require("express");
const app_settings_1 = require("../../../client/deserializers/app-settings");
const app_settings_2 = require("../../../common/models/app-settings/app-settings");
const queryable_data_cube_1 = require("../../../common/models/data-cube/queryable-data-cube");
const sources_1 = require("../../../common/models/sources/sources");
const url_hash_converter_1 = require("../../../common/utils/url-hash-converter/url-hash-converter");
const view_definitions_1 = require("../../../common/view-definitions");
function convertToClientAppSettings(appSettings) {
    return app_settings_1.deserialize(app_settings_2.serialize(appSettings));
}
function convertToClientDataCube(cube) {
    return {
        ...cube,
        timeAttribute: cube.timeAttribute && cube.timeAttribute.name
    };
}
function mkurlRouter(settings) {
    const router = express_1.Router();
    router.post("/", async (req, res) => {
        const { dataCubeName, viewDefinitionVersion, viewDefinition } = req.body;
        if (typeof viewDefinitionVersion !== "string") {
            res.status(400).send({ error: "must have a viewDefinitionVersion" });
            return;
        }
        const definitionConverter = view_definitions_1.definitionConverters[viewDefinitionVersion];
        if (definitionConverter == null) {
            res.status(400).send({ error: "unsupported viewDefinitionVersion value"
            });
            return;
        }
        if (typeof dataCubeName !== "string") {
            res.status(400).send({ error: "must have a dataCubeName" });
            return;
        }
        if (typeof viewDefinition !== "object") {
            res.status(400).send({ error: "viewDefinition must be an object" });
            return;
        }
        let sources;
        try {
            sources = await settings.getSources();
        }
        catch (e) {
            res.status(400).send({ error: "Couldn't load settings" });
            return;
        }
        const myDataCube = sources_1.getDataCube(sources, dataCubeName);
        if (!myDataCube) {
            res.status(400).send({ error: "unknown data cube" });
            return;
        }
        if (!queryable_data_cube_1.isQueryable(myDataCube)) {
            res.status(400).send({ error: "un queryable data cube" });
            return;
        }
        const clientDataCube = convertToClientDataCube(myDataCube);
        const clientAppSettings = convertToClientAppSettings(settings.appSettings);
        let essence;
        try {
            essence = definitionConverter.fromViewDefinition(viewDefinition, clientAppSettings, clientDataCube);
        }
        catch ({ message }) {
            res.status(400).send({ error: "invalid viewDefinition object", message });
            return;
        }
        res.json({
            hash: `#${myDataCube.name}/${url_hash_converter_1.urlHashConverter.toHash(essence)}`
        });
    });
    return router;
}
exports.mkurlRouter = mkurlRouter;
//# sourceMappingURL=mkurl.js.map